// UsbHidPort.cs
//
using System;
using System.ComponentModel;
using System.Collections.Generic;
using System.Diagnostics;
using System.Text;
using System.Drawing;
using System.Windows.Forms;

namespace UsbLibrary
{
  /// <summary>
  /// Usb Component
  /// </summary>
  [ToolboxBitmap(typeof(UsbComponent), "UsbImage.bmp")]
  public partial class UsbComponent : Component
  {
    private UInt32                m_productId;
    private UInt32                m_vendorId;

    private IntPtr                m_usbEventHandle;
    private UsbDevice             m_usbDevice;
    private IntPtr                m_hWnd;

    // Properties
    //
    [Description("The Product Id")]
    [DefaultValue("0000")]
    [Category("Description")]
    [TypeConverter(typeof(UInt32HexTypeConverter))]
    public UInt32 ProductId
    {
      get { return m_productId; }
      set { m_productId = value; }
    }

    [Description("The Vendor Id")]
    [DefaultValue("0000")]
    [Category("Description")]
    [TypeConverter(typeof(UInt32HexTypeConverter))]
    public UInt32 VendorId
    {
      get { return m_vendorId; }
      set { m_vendorId = value; }
    }

    [Browsable(false)]
    public UsbDevice SpecifiedDevice
    {
      get { return this.m_usbDevice; }
    }

    /// <summary>
    /// Constructor
    /// </summary>
    public UsbComponent()
    {
      m_productId = 0;
      m_vendorId = 0;
      m_usbDevice = null;

      InitializeComponent();
    }

    /// <summary>
    /// Constructor
    /// </summary>
    /// <param name="container"></param>
    public UsbComponent(IContainer container)
    {
      m_productId = 0;
      m_vendorId = 0;
      m_usbDevice = null;

      container.Add(this);
      InitializeComponent();
    }

    // Events

    /// <summary>
    /// This event occurs when the device is arrived
    /// </summary>
    [Description("This event occurs when the device is arrived")]
    [Category("Events")]
    [DisplayName("OnDeviceArrived")]
    public event EventHandler     OnDeviceArrived;

    /// <summary>
    /// This event occurs when the device is removed
    /// </summary>
    [Description("This event occurs when the device is removed")]
    [Category("Events")]
    [DisplayName("OnDeviceRemoved")]
    public event EventHandler     OnDeviceRemoved;

    /// <summary>
    /// This event occurs when data is received
    /// </summary>
    [Description("This event occurs when data is received")]
    [Category("Events")]
    [DisplayName("OnDataReceived")]
    public event DataReceivedEventHandler   OnDataReceived;

    /// <summary>
    /// This event occurs when data is send
    /// </summary>
    [Description("This event that occurs when data is send")]
    [Category("Events")]
    [DisplayName("OnDataSend")]
    public event DataSendEventHandler       OnDataSend;

    /// <summary>
    /// Register for Usb Events
    /// </summary>
    /// <param name="hWnd"></param>
    public void RegisterHandle(IntPtr hWnd)
    {
      m_usbEventHandle = Win32Usb.RegisterForUsbEvents(hWnd, Win32Usb.HIDGuid);
      m_hWnd = hWnd;
      CheckDevicePresent();
    }

    /// <summary>
    /// Unregisters for Usb Events  
    /// </summary>
    public bool UnregisterHandle()
    {
      if (m_hWnd != null)
      {
        return Win32Usb.UnregisterForUsbEvents(m_hWnd);
      }          
      return false;
    }

    /// <summary>
    /// Parse Messages
    /// </summary>
    /// <param name="m"></param>
    public void ParseMessages(ref Message m)
    {
      if (m.Msg == Win32Usb.WM_DEVICECHANGE)
      {
        switch (m.WParam.ToInt32())
        {
          case Win32Usb.DEVICE_ARRIVAL:
            CheckDevicePresent();
            break;
          case Win32Usb.DEVICE_REMOVECOMPLETE:
            CheckDevicePresent();
            break;
        }
      }
    }

    /// <summary>
    /// Checks if the device is present 
    /// </summary>
    public void CheckDevicePresent()
    {
      try
      {
        bool bPresent = false;
        if(m_usbDevice != null )
        {
          bPresent = true;
        }

        m_usbDevice = UsbDevice.FindDevice(this.m_vendorId, this.m_productId);	
        if (m_usbDevice != null)
        {
          if (OnDeviceArrived != null)
          {
            this.OnDeviceArrived(this, new EventArgs());
            m_usbDevice.DataReceived += new DataReceivedEventHandler(OnDataReceived);
            m_usbDevice.DataSend += new DataSendEventHandler(OnDataSend);
          }
        }
        else
        {
          if (OnDeviceRemoved != null && bPresent)
          {
            this.OnDeviceRemoved(this, new EventArgs());
          }
        }
      }
      catch (Exception ex)
      {
        Console.WriteLine(ex.ToString());
      }
    }

    private void DataReceived(object sender, DataReceivedEventArgs args)
    {
      if(this.OnDataReceived != null)
      {
        this.OnDataReceived(sender, args);
      }
    }

    private void DataSend(object sender, DataSendEventArgs args)
    {
      if (this.OnDataSend != null)
      {
        this.OnDataSend(sender, args);
      }
    }    
  }
}
