// UsbDevice.cs
//
using System;
using System.Collections.Generic;
using System.IO;
using System.Text;
using System.Runtime.InteropServices;
using Microsoft.Win32.SafeHandles;

namespace UsbLibrary
{
  /// <summary>
  /// Arguments for the OnDataReceived Event
  /// </summary>
  public class DataReceivedEventArgs : EventArgs
  {
    /// <summary>
    /// Received data
    /// </summary>
    public readonly byte[] data;

    /// <summary>
    /// Constructor
    /// </summary>
    /// <param name="data"></param>
    public DataReceivedEventArgs(byte[] data)
    {
      this.data = new byte[data.Length];
      for (int i = 0; i < data.Length; i++)
        this.data[i] = data[i];
    }
  }

  /// <summary>
  /// Arguments for the OnDataSend Event
  /// </summary>
  public class DataSendEventArgs : EventArgs
  {
    /// <summary>
    /// Send Data
    /// </summary>
    public readonly byte[] data;

    /// <summary>
    /// Constructor
    /// </summary>
    /// <param name="data"></param>
    public DataSendEventArgs(byte[] data)
    {
      this.data = new byte[data.Length];
      for (int i = 0; i < data.Length; i++)
        this.data[i] = data[i];
    }
  }

  /// <summary>
  /// Data Received Event Handler
  /// </summary>
  /// <param name="sender"></param>
  /// <param name="args"></param>
  public delegate void DataReceivedEventHandler(object sender, DataReceivedEventArgs args);
  /// <summary>
  /// Data Send Event Handler
  /// </summary>
  /// <param name="sender"></param>
  /// <param name="args"></param>
  public delegate void DataSendEventHandler(object sender, DataSendEventArgs args);

  /// <summary>
  /// Usb Device Exception
  /// </summary>
  public class UsbDeviceException : ApplicationException
  {
    /// <summary>
    /// Constructor
    /// </summary>
    /// <param name="strMessage"></param>
    public UsbDeviceException(string strMessage) : base(strMessage) { }

    /// <summary>
    /// Generate Exception From A Windows Error
    /// </summary>
    /// <param name="strMessage"></param>
    /// <returns></returns>
    public static UsbDeviceException GenerateWithWinError(string strMessage)
    {
      return new UsbDeviceException(string.Format("Msg:{0} WinEr:{1:X8}", strMessage, Marshal.GetLastWin32Error()));
    }

    /// <summary>
    /// Generate Error
    /// </summary>
    /// <param name="strMessage"></param>
    /// <returns></returns>
    public static UsbDeviceException GenerateError(string strMessage)
    {
      return new UsbDeviceException(string.Format("Msg:{0}", strMessage));
    }
  }

  /// <summary>
  /// Usb Device
  /// </summary>
  public class UsbDevice : Win32Usb, IDisposable
  {
    private FileStream            m_oFile;
    private int                   m_nInputReportLength;
    private int                   m_nOutputReportLength;
    private IntPtr                m_hHandle;

    /// <summary>
    /// Data Received Event Handler
    /// </summary>
    public event DataReceivedEventHandler DataReceived;
    /// <summary>
    /// data Send Event Handler
    /// </summary>
    public event DataSendEventHandler     DataSend;
    /// <summary>
    /// Event handler called when device has been removed
    /// </summary>
    public event EventHandler             OnDeviceRemoved;

    /// <summary>
    /// Finds a device given its PID and VID
    /// </summary>
    /// <param name="nVid">Vendor Id</param>
    /// <param name="nPid">Product Id</param>
    /// <returns>A new device class of the given type or null</returns>
    public static UsbDevice FindDevice(UInt32 nVid, UInt32 nPid)
    {
      string strPath = string.Empty;
      string strSearch = string.Format("vid_{0:x4}&pid_{1:x4}", nVid, nPid);
      Guid gHid = HIDGuid;
      IntPtr hInfoSet = SetupDiGetClassDevs(ref gHid, null, IntPtr.Zero, DIGCF_DEVICEINTERFACE | DIGCF_PRESENT);
      try
      {
        DeviceInterfaceData oInterface = new DeviceInterfaceData();	
        oInterface.Size = Marshal.SizeOf(oInterface);
        int nIndex = 0;
        while (SetupDiEnumDeviceInterfaces(hInfoSet, 0, ref gHid, (uint)nIndex, ref oInterface))	
        {
          string strDevicePath = GetDevicePath(hInfoSet, ref oInterface);
          if (strDevicePath.IndexOf(strSearch) >= 0)
          {
            UsbDevice oNewDevice = new UsbDevice();
            oNewDevice.Initialize(strDevicePath);
            return oNewDevice;
          }
          nIndex++;
        }
      }
      catch (Exception ex)
      {
        throw UsbDeviceException.GenerateError(ex.ToString());
      }
      finally
      {
        SetupDiDestroyDeviceInfoList(hInfoSet);
      }

      return null;

    }

    /// <summary>
    /// Get Device Path
     /// </summary>
    /// <param name="hInfoSet">Handle to the InfoSet</param>
    /// <param name="oInterface">DeviceInterfaceData structure</param>
    /// <returns>The device path or null if there was some problem</returns>
    protected static string GetDevicePath(IntPtr hInfoSet, ref DeviceInterfaceData oInterface)
    {
      uint nRequiredSize = 0;
      if (!SetupDiGetDeviceInterfaceDetail(hInfoSet, ref oInterface, IntPtr.Zero, 0, ref nRequiredSize, IntPtr.Zero))
      {
        DeviceInterfaceDetailData oDetail = new DeviceInterfaceDetailData();
        oDetail.Size = 5;	// hardcoded to 5! Sorry, but this works and trying more future proof versions by setting the size to the struct sizeof failed miserably. If you manage to sort it, mail me! Thx
        if (SetupDiGetDeviceInterfaceDetail(hInfoSet, ref oInterface, ref oDetail, nRequiredSize, ref nRequiredSize, IntPtr.Zero))
        {
          return oDetail.DevicePath;
        }
      }
      return null;
    }

    /// <summary>
    /// Handle Data Received
    /// </summary>
    /// <param name="data"></param>
    protected void HandleDataReceived(byte[] data)
    {
      if (DataReceived != null)
      {
        DataReceived(this, new DataReceivedEventArgs(data));
      }
    }

    /// <summary>
    /// Send data
    /// </summary>
    /// <param name="data"></param>
    public void SendData(byte[] data)
    {
      try
      {
        Write(data);
        if (DataSend != null)
        {
          DataSend(this, new DataSendEventArgs(data));
        }
      }
      catch (UsbDeviceException ex)
      {
      }
      catch (Exception ex)
      {
        Console.WriteLine(ex.ToString());
      }
    }

    /// <summary>
    /// Dispose
    /// </summary>
    public void Dispose()
    {
      Dispose(true);
      GC.SuppressFinalize(this);
    }

    /// <summary>
    /// Dispose
    /// </summary>
    /// <param name="bDisposing"></param>
    protected virtual void Dispose(bool bDisposing)
    {
      try
      {
        if (bDisposing)
        {
          if (m_oFile != null)
          {
            m_oFile.Close();
            m_oFile = null;
          }
        }
        if (m_hHandle != IntPtr.Zero)	
        {

          CloseHandle(m_hHandle);
        }
      }
      catch (Exception ex)
      {
        Console.WriteLine(ex.ToString());
      }
    }

    /// <summary>
    /// Initialize
    /// </summary>
    /// <param name="strPath"></param>
		protected void Initialize(string strPath)
		{
      m_hHandle = CreateFile(strPath, GENERIC_READ | GENERIC_WRITE, 0, IntPtr.Zero, OPEN_EXISTING, FILE_FLAG_OVERLAPPED, IntPtr.Zero);
      if (( m_hHandle != InvalidHandleValue) && (m_hHandle != null))
			{
				IntPtr lpData;
				if (HidD_GetPreparsedData(m_hHandle, out lpData))
				{
          try
          {
            HidCaps oCaps;
            HidP_GetCaps(lpData, out oCaps);
            m_nInputReportLength = oCaps.InputReportByteLength;
            m_nOutputReportLength = oCaps.OutputReportByteLength;
            m_oFile = new FileStream(new SafeFileHandle(m_hHandle, false), FileAccess.Read | FileAccess.Write, m_nInputReportLength, true);
            BeginAsyncRead();	                         
          }
          catch (Exception ex)
          {
            throw UsbDeviceException.GenerateWithWinError("Failed to get the detailed data from the hid.");
          }
  				finally
					{
						HidD_FreePreparsedData(ref lpData);
					}
				}
				else
				{
					throw UsbDeviceException.GenerateWithWinError("GetPreparsedData failed");
				}
			}
			else
			{
				m_hHandle = IntPtr.Zero;
				throw UsbDeviceException.GenerateWithWinError("Failed to create device file");
			}
		}

		/// <summary>
		/// Asynchronous read
		/// </summary>
    private void BeginAsyncRead()
    {
      byte[] arrInputReport = new byte[m_nInputReportLength];
      m_oFile.BeginRead(arrInputReport, 0, m_nInputReportLength, new AsyncCallback(ReadCompleted), arrInputReport);
    }
		/// <summary>
		/// Call for Asynchronous read completed
		/// </summary>
		/// <param name="iResult">Async result parameter</param>
    protected void ReadCompleted(IAsyncResult iResult)
    {
      byte[] arrBuff = (byte[])iResult.AsyncState;
      try
      {
        m_oFile.EndRead(iResult);	
        try
        {
          HandleDataReceived(arrBuff);
        }
        finally
        {
          BeginAsyncRead();
        }                
      }
      catch(IOException ex)	
      {
        if (OnDeviceRemoved != null)
        {
          OnDeviceRemoved(this, new EventArgs());
        }
        Dispose();
      }
    }

		/// <summary>
		/// Write data to the device.
		/// </summary>
		/// <param name="data">data to write</param>
    protected void Write(byte[] data)
    {
      try
      {
        m_oFile.Write(data, 0, data.Length);
      }
      catch (IOException ex)
      {
        throw new UsbDeviceException("Probably the device was removed");
      }
	  	catch(Exception exx)
			{
        Console.WriteLine(exx.ToString());	
		  }
    }

		/// <summary>
		/// Accessor for output report length
		/// </summary>
		public int OutputReportLength
		{
			get
			{
				return m_nOutputReportLength;
			}
		}
		/// <summary>
		/// Accessor for input report length
		/// </summary>
		public int InputReportLength
		{
			get
			{
				return m_nInputReportLength;
			}
		}
   }
}
